#!/usr/bin/env python3

import os
import time
import subprocess
import uuid
import nvr

# Helper functions

def run_nvim(env):
    nvim = subprocess.Popen(['nvim', '-nu', 'NORC', '--headless'], env=env)
    time.sleep(1)
    return nvim

def run_nvr(cmdlines, env):
    for cmdline in cmdlines:
        nvr.main(cmdline, env)

def setup_env():
    env = {'NVIM_LISTEN_ADDRESS': 'pytest_socket_{}'.format(uuid.uuid4())}
    env.update(os.environ)
    return env

# Tests

def test_remote_send(capsys):
    env = setup_env()
    nvim = run_nvim(env)
    cmdlines = [['nvr', '--nostart', '--remote-send', 'iabc<cr><esc>'],
                ['nvr', '--nostart', '--remote-expr', 'getline(1)']]
    run_nvr(cmdlines, env)
    nvim.terminate()
    out, err = capsys.readouterr()
    assert out == 'abc\n'

# https://github.com/mhinz/neovim-remote/issues/77
def test_escape_filenames_properly(capsys):
    filename = 'a b|c'
    env = setup_env()
    nvim = run_nvim(env)
    cmdlines = [['nvr', '-s', '--nostart', '-o', filename],
                ['nvr', '-s', '--nostart', '--remote-expr', 'fnamemodify(bufname(""), ":t")']]
    run_nvr(cmdlines, env)
    nvim.terminate()
    out, err = capsys.readouterr()
    assert filename == out.rstrip()

def test_escape_single_quotes_in_filenames(capsys):
    filename = "foo'bar'quux"
    env = setup_env()
    nvim = run_nvim(env)
    cmdlines = [['nvr', '-s', '--nostart', '-o', filename],
                ['nvr', '-s', '--nostart', '--remote-expr', 'fnamemodify(bufname(""), ":t")']]
    run_nvr(cmdlines, env)
    nvim.terminate()
    out, err = capsys.readouterr()
    assert filename == out.rstrip()

def test_escape_double_quotes_in_filenames(capsys):
    filename = 'foo"bar"quux'
    env = setup_env()
    nvim = run_nvim(env)
    cmdlines = [['nvr', '-s', '--nostart', '-o', filename],
                ['nvr', '-s', '--nostart', '--remote-expr', 'fnamemodify(bufname(""), ":t")']]
    run_nvr(cmdlines, env)
    nvim.terminate()
    out, err = capsys.readouterr()
    assert filename == out.rstrip()



from setuptools import setup
from os         import path

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name             = 'neovim-remote',
    author           = 'Marco Hinz',
    author_email     = 'mh.codebro@gmail.com',
    url              = 'https://github.com/mhinz/neovim-remote',
    description      = 'Control nvim processes using "nvr" commandline tool',
    long_description = long_description,
    long_description_content_type = 'text/markdown',
    python_requires  = '>=3.4',
    install_requires = ['pynvim', 'psutil', 'setuptools'],
    entry_points     = {
        'console_scripts': ['nvr = nvr.nvr:main']
    },
    packages         = ['nvr'],
    version          = '2.2.3',
    license          = 'MIT',
    keywords         = 'neovim nvim nvr remote helper',
    classifiers      = [
        'Topic :: Utilities',
        'License :: OSI Approved :: MIT License',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: End Users/Desktop',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
)

